/*
 * Project: MoleCuilder
 * Description: creates and alters molecular systems
 * Copyright (C)  2010-2012 University of Bonn. All rights reserved.
 * 
 *
 *   This file is part of MoleCuilder.
 *
 *    MoleCuilder is free software: you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation, either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    MoleCuilder is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with MoleCuilder.  If not, see <http://www.gnu.org/licenses/>.
 */

/*
 * CalculateMolarMassAction.cpp
 *
 *  Created on: Nov 02, 2011
 *      Author: heber
 */

// include config.h
#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include "CodePatterns/MemDebug.hpp"

#include "CodePatterns/Log.hpp"

#include "Atom/atom.hpp"
#include "Element/element.hpp"
#include "World.hpp"

#include <iostream>
#include <string>

#include "CalculateMolarMassAction.hpp"

using namespace MoleCuilder;

// and construct the stuff
#include "CalculateMolarMassAction.def"
#include "Reaction_impl_pre.hpp"

#include "Reaction_impl.hpp"

/** =========== define the function ====================== */
double * AnalysisCalculateMolarMassAction::doCalc() {
  double mass = 0.;
  size_t totalnumber = 0;
  // calculate volume from Box
  for(World::AtomSelectionConstIterator iter = World::getInstance().beginAtomSelection();
      iter != World::getInstance().endAtomSelection();
      ++iter) {
    mass += iter->second->getType()->getMass();
    ++totalnumber;
  }
  LOG(0, "RESULT: The mass of " << totalnumber << " selected atoms is " << mass << " atomicmassunit.");

  return (new double(mass));
}
/** =========== end of function ====================== */
